@@ -3,14 +3,10 @@ package Pod::Xhtml;
 use strict;
 use Pod::Parser;
 use Pod::ParseUtils;
-use Carp;
-use vars qw/@ISA %COMMANDS %SEQ $VERSION $FirstAnchorId $ContentSuffix/;
-
-$FirstAnchorId = "TOP";
-$ContentSuffix = "-CONTENT";
+use vars qw/@ISA %COMMANDS %SEQ $VERSION/;
 
 @ISA = qw(Pod::Parser);
-$VERSION = '1.52_01'; # unofficial jrockway release
+($VERSION) = ('$Revision: 1.35 $' =~ m/([\d\.]+)/);
 
 # recognized commands
 %COMMANDS = map { $_ => 1 } qw(pod head1 head2 head3 head4 item over back for begin end);
@@ -54,15 +50,12 @@ sub parse_from_filehandle {
 sub initialize {
 	my $self = shift;
 
-	$self->{TopLinks} = qq(<p><a href="#<<<G?$FirstAnchorId>>>" class="toplink">Top</a></p>) unless defined $self->{TopLinks};
+	$self->{TopLinks} = qq(<p><a href="#TOP" class="toplink">Top</a></p>) unless defined $self->{TopLinks};
 	$self->{MakeIndex} = 1 unless defined $self->{MakeIndex};
 	$self->{MakeMeta} = 1 unless defined $self->{MakeMeta};
 	$self->{FragmentOnly} = 0 unless defined $self->{FragmentOnly};
 	$self->{HeadText} = $self->{BodyOpenText} = $self->{BodyCloseText} = '';
 	$self->{LinkParser} ||= new Pod::Hyperlink;
-	$self->{TopHeading} ||= 1;
-	$self->{TopHeading} = int $self->{TopHeading}; # heading level must be an integer
-	croak "TopHeading must be greater than zero" if $self->{TopHeading} < 1; # (prevent negative heading levels)
 	$self->SUPER::initialize();
 }
 
@@ -106,23 +99,12 @@ sub end_pod {
 
 	# now loop over each para and expand any html escapes or sequences
 	$self->_paraExpand( $_ ) foreach (@$ptree);
-
-	$self->{buffer} =~ s/(\n?)<\/pre>\s*<pre>/$1/sg; # concatenate 'pre' blocks
-	1 while $self->{buffer} =~ s/<pre>(\s+)<\/pre>/$1/sg;
+	$self->{buffer} =~ s/\n?<\/pre>(\s*)<pre>/$1/sg; # concatenate 'pre' blocks
+	$self->{buffer} =~ s/<pre>\s+<\/pre>//sg;
 	$self->{buffer} = $self->_makeIndex . $self->{buffer} if $self->{MakeIndex};
-	$self->{buffer} =~ s/<<<G\?($FirstAnchorId)>>>/$1/ge;
-	$self->{buffer} = join "\n", qq[<div class="pod">], $self->{buffer},
-		( @{ $self->{sections} } > 1 && "</div>" ), "</div>";
+	$self->{buffer} = qq(<a name="TOP"></a>) . $self->{buffer};
 
-	# Expand internal L<> links to the correct sections
-	$self->{buffer} =~ s/#<<<(.*?)>>>/'#' . $self->_findSection($1)/eg;
-	die "gotcha" if $self->{buffer} =~ /#<<</;
-
-	my $headblock = sprintf "%s\n%s\n%s\n\t<title>%s</title>\n",
-		qq(<?xml version="1.0" encoding="UTF-8"?>),
-		qq(<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">),
-		qq(<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">\n<head>),
-		_htmlEscape( $self->{doctitle} );
+	my $headblock = qq(<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">\n<head>\n\t<title>) . _htmlEscape( $self->{doctitle} ) . "</title>\n";
 	$headblock .= $self->_makeMeta if $self->{MakeMeta};
 
 	unless ($self->{FragmentOnly}) {
@@ -145,16 +127,9 @@ sub resetMe {
 	$self->{'sections'} = [];
 	$self->{'listKind'} = [];
 	$self->{'listHasItems'} = [];
-	$self->{'dataSections'} = [];
-	$self->{'section_names'} = {};
-	$self->{'section_ids'} = {};
-	$self->{'tagStack'} = [];
 
 	foreach (qw(inList titleflag )) { $self->{$_} = 0; }
 	foreach (qw(buffer doctitle)) { $self->{$_} = ''; }
-	
-	# add the "$FirstAnchor" section into the sections
-	$self->_addSection ( '', $FirstAnchorId );
 }
 
 sub parse_tree { return $_[0]->{'-ptree'}; }
@@ -164,25 +139,17 @@ sub _paraExpand {
 	my $para = shift;
 
 	# collapse interior sequences and strings
-	foreach ( @{$para->{'-ptree'}} ) {
-		$_ = (ref $_) ? $self->_handleSequence($_) : _htmlEscape( $_ );
-	}
+	foreach ( @{$para->{'-ptree'}} ) { $_ = (ref $_) ? $self->_handleSequence($_) : _htmlEscape( $_ ); }
 
 	# the parse tree has now been collapsed into a list of strings
 	if ($para->{TYPE} eq 'TEXT') {
-		return if @{$self->{dataSections}};
 		$self->_addTextblock( join('', @{$para->{'-ptree'}}) );
 	} elsif ($para->{TYPE} eq 'VERBATIM') {
-		return if @{$self->{dataSections}};
-		my $paragraph = "<pre>" . join('', @{$para->{'-ptree'}}) . "\n\n</pre>";
-		# tell _addTextblock NOT to put this in a paragraph block
-		$self->_addTextblock( $paragraph );
-		if ($self->{titleflag} != 0) {
-			$self->_setTitle( $paragraph );
-			warn "NAME followed by verbatim paragraph";
-		}
+		my $paragraph = join('', @{$para->{'-ptree'}});
+		$self->{buffer} .= "<pre>$paragraph\n\n</pre>\n";
+		if ($self->{titleflag} != 0) { $self->_setTitle( $paragraph ); warn "NAME followed by verbatim paragraph"; }
 	} elsif ($para->{TYPE} eq 'COMMAND') {
-		$self->_addCommand($para->{'-name'}, join('', @{$para->{'-ptree'}}), $para->{'-text'}, $para->{'-line'} )
+		$self->_addCommand( $para->{'-name'}, join('', @{$para->{'-ptree'}}), $para->{'-line'} )
 	} else {
 		warn "Unrecognized paragraph type $para->{TYPE} found at $self->{_INFILE} line $para->{'-line'}\n";
 	}
@@ -190,8 +157,7 @@ sub _paraExpand {
 
 sub _addCommand {
 	my $self = shift;
-	my ($command, $paragraph, $raw_para, $line) = @_;
-	my $anchor;
+	my ($command, $paragraph, $line) = @_;
 
 	unless (exists $COMMANDS{$command}) {
 		warn "Unrecognized command '$command' skipped at $self->{_INFILE} line $line\n";
@@ -199,40 +165,20 @@ sub _addCommand {
 	}
 
 	for ($command) {
-		my $data_para = @{$self->{dataSections}}; # inside a data paragraph?
-		/^head1/ && !$data_para && do {
-		        my $top_heading = 'h'. $self->{TopHeading};
-			$top_heading = 'h1' if !$self->{FragmentOnly}; # ignore TopHeading when not in fragment mode
-
-			# if ANY sections are open then close the previously opened div
-			$self->{buffer} .= "\n</div>\n" unless ( @{ $self->{sections} } == 1 );
-			
-			$anchor = $self->_addSection( 'head1', $paragraph );
-			my $anchorContent = $self->_addSection( '', $paragraph . $ContentSuffix);
-			
-			$self->{buffer} .= qq(<$top_heading id="$anchor">$paragraph</$top_heading>)
-					.($self->{TopLinks} ? $self->{TopLinks} : '')."\n"
-					."<div id=\"$anchorContent\">\n";
-
+		/^head1/ && do {
+			my $anchor = $self->_addIndex( 'head1', $paragraph );
+			$self->{buffer} .= qq(<h1><a name="$anchor"></a>$paragraph</h1>)
+					.($self->{TopLinks} ? $self->{TopLinks} : '')."\n\n";
 			if ($anchor eq 'NAME') { $self->{titleflag} = 1; }
 			last;
 		};
-		/^head([234])/ && !$data_para && do {
+		/^head([234])/ && do {
 			my $head_level = $1;
-			if($self->{FragmentOnly}){
-			    $head_level += ($self->{TopHeading} - 1);
-			    $head_level = 6 if $head_level > 6;
-			}
-			# if ANY sections are open then close the previously opened div
-			$self->{buffer} .= "\n</div>\n" unless ( @{ $self->{sections} } == 1 );
-
-			$anchor = $self->_addSection( "head${head_level}", $paragraph );
-			my $anchorContent = $self->_addSection( '', $paragraph . $ContentSuffix);
-
-			$self->{buffer} .= "<h${head_level} id=\"$anchor\">$paragraph</h${head_level}>\n" . "<div id=\"$anchorContent\">\n";
+			my $anchor = $self->_addIndex( "head${head_level}", $paragraph );
+			$self->{buffer} .= qq(<h${head_level}><a name="$anchor"></a>$paragraph</h${head_level}>\n\n);
 			last;
 		};
-		/^item/ && !$data_para && do {
+		/^item/ && do {
 			unless ($self->{inList}) {
 				warn "Not in list at $self->{_INFILE} line $line\n";
 				last;
@@ -243,64 +189,37 @@ sub _addCommand {
 
 			# is this the first item in the list?
 			if (@{$self->{listKind}} && $self->{listKind}[-1] == 0) {
-				my $parent_list = $self->{listKind}[-2]; # this is a sub-list
-				if ($parent_list && $parent_list == 1) {
-					# <ul> sub lists must be in an <li> [BEGIN]
-					$self->{buffer} .= $self->_tagLevel () . "<li>\n";
-					push @{$self->{tagStack}}, "li";
-				} elsif ($parent_list && $parent_list == 2) {
-					# <dl> sub lists must be in a <p> [BEGIN]
-					$self->{buffer} .= $self->_tagLevel () . "<p>\n";
-					push @{$self->{tagStack}}, "p";
-				}
-
 				if ($paragraph eq '*') {
 					$self->{listKind}[-1] = 1;
-					$self->{buffer} .= $self->_tagLevel () . "<ul>\n";
-					push @{$self->{tagStack}}, "ul";
+					$self->{buffer} .= "<ul>\n";
 				} else {
 					$self->{listKind}[-1] = 2;
-					$self->{buffer} .= $self->_tagLevel () . "<dl>\n";
-					push @{$self->{tagStack}}, "dl";
+					$self->{buffer} .= "<dl>\n";
 				}
 			} else {
 				# close last list item's tag#
 				if ($self->{listKind}[-1] == 1) {
-					my $o = pop @{$self->{tagStack}};
-					warn "expected 'li' to be on the tag stack but got '$o'\n"
-						if $o ne 'li';
-					$self->{buffer} .= $self->_tagLevel () . "</li>\n";
+					$self->{buffer} .= "</li>\n";
 				}
 			}
 			if (@{$self->{listKind}} && $self->{listKind}[-1] == 2) {
-				if (@{$self->{tagStack}} && $self->{tagStack}[-1] eq "dd") {
-					my $o = pop @{$self->{tagStack}};
-					warn "expected 'dd' to be on the tag stack but got '$o'\n"
-						if $o ne 'dd';
-					$self->{buffer} .= $self->_tagLevel () . "</dd>\n";
-				}
-				$self->{buffer} .= $self->_tagLevel () . qq(<dt);
-				push @{$self->{tagStack}}, "dt";
+				$self->{buffer} .= qq(\t<dt>);
 				if ($self->{MakeIndex} >= 2) {
-					$anchor = $self->_addSection( 'item', $paragraph );
-					$self->{buffer} .= qq( id="$anchor");
+					my $anchor = $self->_addIndex( 'list', $paragraph );
+					$self->{buffer} .= qq(<a name="$anchor"></a>);
 				}
-				$self->{buffer} .= ">";
 				$self->{buffer} .= qq($paragraph</dt>\n);
-				my $o = pop @{$self->{tagStack}};
-				warn "expected 'dt' to be on the tag stack but got '$o'\n"
-					if $o ne 'dt';
 			}
 			last;
 		};
-		/^over/ && !$data_para && do {
+		/^over/ && do {
 			$self->{inList}++;
 			push @{$self->{listKind}}, 0;
 			push @{$self->{listHasItems}}, 0;
-			push @{$self->{sections}}, 'over' if $self->{MakeIndex} >= 2;
+			push @{$self->{sections}}, 'OVER';
 			push @{$self->{listCurrentParas}}, 0;
 		};
-		/^back/ && !$data_para && do {
+		/^back/ && do {
 			if (--$self->{inList} < 0) {
 				warn "=back commands don't balance =overs at $self->{_INFILE} line $line\n";
 				last;
@@ -308,78 +227,18 @@ sub _addCommand {
 				warn "empty list at $self->{_INFILE} line $line\n";
 				last;
 			} elsif (@{$self->{listKind}} && $self->{listKind}[-1] == 1) {
-				my $o = pop @{$self->{tagStack}};
-				warn "expected 'li' to be on the tag stack but got '$o'\n"
-					if $o ne 'li';
-				$o = pop @{$self->{tagStack}};
-				warn "expected 'ul' to be on the tag stack but got '$o'\n"
-					if $o ne 'ul';
-				$self->{buffer} .= "</li>\n</ul>\n\n";
+				$self->{buffer} .= "</li></ul>\n\n";
 			} else {
-				while (@{$self->{tagStack}} && $self->{tagStack}[-1] eq "dd") {
-					pop @{$self->{tagStack}};
-					$self->{buffer} .=$self->_tagLevel () . "</dd>\n";
-				}
-				my $o = pop @{$self->{tagStack}};
-				warn "expected 'dl' to be on the tag stack but got '$o'\n"
-					if $o ne 'dl';
-				$self->{buffer} .= $self->_tagLevel () . "</dl>\n";
+				$self->{buffer} .= "</dl>\n\n";
 			}
-
-			my $parent_list = $self->{listKind}[-2]; # this is a sub-list
-			if ($parent_list && $parent_list == 1) {
-				my $o = pop @{$self->{tagStack}};
-				warn "expected 'li' to be on the tag stack but got '$o'\n"
-					if $o ne 'li';
-				# <ul> sub lists must be in an <li> [END]
-				$self->{buffer} .= $self->_tagLevel () . "</li>\n";
-			}
-			if ($parent_list && $parent_list == 2) {
-				my $o = pop @{$self->{tagStack}};
-				warn "expected 'p' to be on the tag stack but got '$o'\n"
-					if $o ne 'p';
-				# <dl> sub lists must be in a <p> [END]
-				$self->{buffer} .= $self->_tagLevel () . "</p>\n";
-			}
-
-			if ( $self->{MakeIndex} >= 2 ) {
-				if ( ! ref $self->{sections}->[ -1 ] ) {
-					if ( $self->{sections}->[ -1 ] =~ /^over$/i ) {
-						pop @{ $self->{sections} };
-					}
-				} else {
-					if ( $self->{sections}->[ -1 ] [ 0 ] =~ /^item$/i ) {
-						push @{ $self->{sections} }, 'back';
-					}
-				}
-			}
-
-			pop @{$self->{listHasItems}};
-			pop @{$self->{listKind}};
-			pop @{$self->{listCurrentParas}};
+			push @{$self->{sections}}, 'BACK';
+			pop  @{$self->{listHasItems}};
+			pop  @{$self->{listKind}};
+			pop  @{$self->{listCurrentParas}};
 			last;
 		};
-		/^for/ && !$data_para && do {
-			my ($html) = $raw_para =~ /^\s*(?:pod2)?x?html\s+(.*)/;
-			$self->{buffer} .= $html if $html;
-		};
-		/^begin/ && !$data_para && do {
-			my ($ident) = $paragraph =~ /(\S+)/;
-			push @{$self->{dataSections}}, $ident;
-			last;
-		};
-		/^end/ && do {
-			my ($ident) = $paragraph =~ /(\S+)/;
-			unless (@{$self->{dataSections}}) {
-				warn "no corresponding '=begin $ident' marker at $self->{_INFILE} line $line\n";
-				last;
-			}
-			my $current_section = $self->{dataSections}[-1];
-			unless ($current_section eq $ident) {
-				warn "'=end $ident' doesn't match '=begin $current_section' at $self->{_INFILE} line $line\n";
-				last;
-			}
-			pop @{$self->{dataSections}};
+		/^for/ || /^begin/ || /^end/ && do {
+			warn "COMMAND $_ UNIMPLEMENTED AT THE MOMENT! at $self->{_INFILE} line $line\n";
 			last;
 		};
 	}
@@ -392,39 +251,18 @@ sub _addTextblock {
 	if ($self->{titleflag} != 0) { $self->_setTitle( $paragraph ); }
 
 	if (! @{$self->{listKind}} || $self->{listKind}[-1] == 0) {
-		# DON'T put a paragraph in a <p> if it's a <pre>!
-		if ($paragraph !~ m/^\s*<pre>/im) {
-			$self->{buffer} .= $self->_tagLevel () . "<p>$paragraph</p>\n";
-		} else {
-			$self->{buffer} .= "$paragraph\n";
-		}
+		$self->{buffer} .= "<p>$paragraph</p>\n\n";
 	} elsif (@{$self->{listKind}} && $self->{listKind}[-1] == 1) {
 		if ($self->{listCurrentParas}[-1]++ == 0) {
-			# should this list item be closed?
-			push @{$self->{tagStack}}, "li";
-			$self->{buffer} .= $self->_tagLevel () . "<li>$paragraph";
+			$self->{buffer} .= "\t<li>$paragraph";
 		} else {
 			$self->{buffer} .= "\n<br /><br />$paragraph";
 		}
 	} else {
-		if ($self->{listCurrentParas}[-1]++ == 0) {
-			$self->{buffer} .= $self->_tagLevel () . "<dd>\n";
-			push @{$self->{tagStack}}, "dd";
-		}
-
-		if ($paragraph !~ m/^\s*<pre>/im) {
-			$self->{buffer} .= $self->_tagLevel () . "<p>$paragraph</p>\n";
-		} else {
-			$self->{buffer} .= "$paragraph\n";
-		}
+		$self->{buffer} .= "\t\t<dd>$paragraph</dd>\n";
 	}
 }
 
-sub _tagLevel {
-	my $self = shift;
-	return ( "\t" x scalar @{$self->{tagStack}} );
-}
-
 # expand interior sequences recursively, bottom up
 sub _handleSequence {
 	my $self = shift;
@@ -435,7 +273,7 @@ sub _handleSequence {
 		if (ref $_) {
 			$buffer .= $self->_handleSequence($_);
 		} else {
-			$buffer .= _htmlEscape($_);
+			$buffer .= $_;
 		}
 	}
 
@@ -446,147 +284,41 @@ sub _handleSequence {
 	return $SEQ{$seq->{'-name'}}->($self, $buffer);
 }
 
-sub _makeIndexId {
+sub _makeIndexName {
 	my $arg = shift;
 
-	$arg =~ s/\W+/_/g;
-	$arg =~ s/^_+|_+$//g;
-	$arg =~ s/__+/_/g;
+	$arg =~ s/&\w+?;/_/g;
 	$arg = substr($arg, 0, 36);
+	$arg =~ tr/a-zA-Z0-9_/_/c;
 	return $arg;
 }
 
-sub _addSection {
+sub _addIndex {
 	my $self = shift;
 	my ($type, $htmlarg) = @_;
 	return unless defined $htmlarg;
 
-	my $index_id;
-	if ($self->{section_names}{$htmlarg}) {
-		$index_id = $self->{section_names}{$htmlarg};
-	} else {
-		$index_id = _makeIndexId($htmlarg);
-	}
-	
-	if ($self->{section_ids}{$index_id}++) {
-		$index_id .= "-" . $self->{section_ids}{$index_id};
-	}
-	
-	# if {section_names}{$htmlarg} is already set then this is a duplicate 'id',
-	# so keep the reference to the first one
-	$self->{section_names}{$htmlarg} = $index_id
-		unless exists $self->{section_names}{$htmlarg};
-
-	push( @{$self->{sections}}, [$type, $index_id, $htmlarg]);
-	return $index_id;
-}
-
-sub _findSection {
-	my $self = shift;
-	my ($htmlarg) = @_;
-
-	my $index_id;
-	if ($index_id = $self->{section_names}{$htmlarg}) {
-		return $index_id;
-	} else {
-		return _makeIndexId($htmlarg);
-	}
-}
-
-sub _get_elem_level {
-	my $elem = shift;
-	if (ref($elem)) {
-		my $type = $elem->[0];
-		if ($type =~ /^head(\d+)$/) {
-			return $1;
-		} else {
-			return 0;
-		}
-	} else {
-		return 0;
-	}
-}
-
-sub _makeTabbing {
-	my $level = shift || 0;
-
-	return "\n" . ( "\t" x $level );
+	my $arg = _makeIndexName($htmlarg);
+	push( @{$self->{sections}}, [$type, $arg, $htmlarg]);
+	return $arg;
 }
 
 sub _makeIndex {
 	my $self = shift;
-
-	my $string = "<!-- INDEX START -->\n<h3 id=\"$FirstAnchorId\">Index</h3>\n";
-
-	my $previous_level = 0;
-	my $previous_section = '';
-	
-	SECTION: foreach my $section ( @{ $self->{sections} } )
-	{
-		my $this_level = 0;
-
-		if ( ! ref $section )
-		{
-			for ( $section )
-			{
-				if ( $section =~ m/^over$/i )
-				{
-					$previous_level++;
-					$string .= ( $previous_section ne 'over' && "</li>\n" ) .
-						"<li>\n<ul>\n";
-				}
-				elsif ( $section =~ m/^back$/i )
-				{
-					$previous_level--;
-					$string .= "\n</li>\n</ul>";
-				}
-			}
-
-			$previous_section = $section;
-		}
-		else
-		{
-			my ( $type, $href, $name ) = @$section;
-			
-			if ( $section->[ 0 ] =~ m/^item$/i )
-			{
-				$this_level = $previous_level;
-			}
-			else
-			{
-				$this_level = _get_elem_level ( $section );
-			}
-
-			next SECTION if $this_level == 0;
-			
-			if ( $this_level > $previous_level )
-			{
-				# open new list(s)
-				$string .= "\n<ul>" .
-					( "\n<li>\n<ul>" ) x ( $this_level - $previous_level - 1 );
-			}
-			elsif ( $this_level < $previous_level )
-			{
-				# close list(s)
-				$string .= "</li>\n" .
-					( "</ul>\n</li>\n" ) x ( $previous_level - $this_level );
-			}
-			else
-			{
-				$string .= "</li>\n" unless $previous_section =~ /^over$/i;
-			}
-
-			$string .= '<li><a href="#' . $href . '">' . $name . '</a>';
-
-			$previous_level = $this_level;
-
-			$previous_section = ( ref $section ? $section->[ 0 ] : $section );
+	my $string = "<!-- INDEX START -->\n<h3>Index</h3>\n<ul>\n";
+	foreach ( @{$self->{sections}} ) {
+		if (ref $_) {
+			my ($type, $href, $name) = @$_;
+			my $index_link = qq(\t<li><a href="#${href}">${name}</a></li>);
+			$index_link = qq(<ul>$index_link</ul>) unless ($type eq 'head1');
+			$string .= $index_link;
+		} elsif ($_ eq 'OVER') {
+			$string .= qq(\t<ul>\n);
+		} elsif ($_ eq 'BACK') {
+			$string .= qq(\t</ul>\n);
 		}
 	}
-
-	$string .= ( "\n</li>\n</ul>" x $previous_level );
-	$string .= "<hr />\n<!-- INDEX END -->\n\n";
-	
+	$string .= "</ul><hr />\n<!-- INDEX END -->\n\n";
 	return $string;
 }
 
@@ -636,23 +368,22 @@ sub seqL {
 	my ($self, $link) = @_;
 	$self->{LinkParser}->parse( $link );
 
-	my $page = $self->{LinkParser}->page;
+	my $page = _htmlEscape( $self->{LinkParser}->page );
 	my $kind = $self->{LinkParser}->type;
 	my $string = '';
 
 	if ($kind eq 'hyperlink') {	#easy, a hyperlink
-		my $targ = $self->{LinkParser}->node;
-		my $text = $self->{LinkParser}->text;
+		my $targ = _htmlEscape( $self->{LinkParser}->node );
+		my $text = _htmlEscape( $self->{LinkParser}->text );
 		$string = qq(<a href="$targ">$text</a>);
 	} elsif ($page eq '') {	# a link to this page
-		# Post-process these links so we can things up to the correct sections
-		my $targ = $self->{LinkParser}->node;
+		my $targ = _htmlEscape( _makeIndexName( $self->{LinkParser}->node ) );
 		$string = $self->{LinkParser}->markup;
-		$string =~ s|Q<(.+?)>|<a href="#<<<$targ>>>">$1</a>|;
+		$string =~ s|Q<(.+?)>|qq(<a href="#$targ">) . _htmlEscape( $1 ) . '</a>'|e;
 	} elsif ($link !~ /\|/) {	# a link off-page with _no_ alt text
 		$string = $self->{LinkParser}->markup;
-		$string =~ s|Q<(.+?)>|<b>$1</b>|;
-		$string =~ s|P<(.+?)>|<cite>$1</cite>|;
+		$string =~ s|Q<(.+?)>|'<b>' . _htmlEscape( $1 ) . '</b>'|e;
+		$string =~ s|P<(.+?)>|'<cite>' . _htmlEscape( $1 ) . '</cite>'|e;
 	} else {	# a link off-page with alt text
 		my $text = _htmlEscape( $self->{LinkParser}->text );
 		my $targ = _htmlEscape( $self->{LinkParser}->node );
@@ -672,8 +403,8 @@ sub seqS {
 sub seqX {
 	my $self = shift;
 	my $arg = shift;
-	my $anchor = $self->_addSection( 'head1', $arg );
-	return qq[<span id="$anchor">$arg</span>];
+	my $anchor = $self->_addIndex( 'head1', $arg );
+	return qq[<a name="$anchor">$arg</a>];
 }
 
 sub seqE {
@@ -807,21 +538,6 @@ Default: 0. If 1, we only produce an XHTML fragment (suitable for use as a
 server-side include etc). There is no HEAD element nor any BODY or HTML
 tags. Any text added with the add*Text methods will B<not> be output.
 
-
-=item TopHeading
-
-Allows you to set the starting heading level when in fragment mode.
-For example, if your document already has h1 tags and you want the
-generated POD to nest inside the outline, you can specify
-
-     TopHeading => 2
-
-and C<=head1> will be tagged with h2 tags, C<=head3> with h3, and so
-on.
-
-Note that XHTML doesn't allow for heading tags past h6, so h7 and up
-will be translated to h6 as necessary.
-
 =item TopLinks
 
 At each section head this text is added to provide a link back to the top.
@@ -871,11 +587,9 @@ P Kent E<amp> Simon Flack  E<lt>cpan _at_ bbc _dot_ co _dot_ ukE<gt>
 
 =head1 COPYRIGHT
 
-(c) BBC 2004, 2005. This program is free software; you can redistribute it
-and/or modify it under the GNU GPL.
+(c) BBC 2004. This program is free software; you can redistribute it and/or
+modify it under the GNU GPL.
 
 See the file COPYING in this distribution, or http://www.gnu.org/licenses/gpl.txt
 
 =cut
-
-# vim:noet
